@@ -40,8 +40,6 @@
 #endif
 #ifdef HAVE_XFT
 #include <X11/Xft/Xft.h>
-#endif
-#ifdef HAVE_FREETYPE
 #include <ft2build.h>
 #include FT_FREETYPE_H
 #include FT_OUTLINE_H
@@ -114,6 +112,34 @@ struct SSysFontInfo {
             return 4;
         }
     }
+    static unsigned long UTF8toUTF32 (const char* utf8, unsigned char *nBytes) {
+        unsigned char arr[4];
+        memset(arr, 0, 4);
+        *nBytes = UTF8codepointBytes(*utf8);
+        memcpy(arr + 4-*nBytes, utf8, *nBytes);
+        arr[4-*nBytes] &= 255 >> *nBytes;
+        return ((arr[0] & 63) << 18  |  (arr[1] & 63) << 12  |
+                (arr[2] & 63) << 6  |  (arr[3] & 127));
+    }
+
+    double GetStrWidth(const char *str) const {
+        double w = 0;
+        unsigned int length = strlen(str);
+        unsigned char len;
+        unsigned long prevCh, nextCh;
+        nextCh = UTF8toUTF32(str, &len);
+        unsigned int i;
+        for (i = len;  i < length;  i += len) {
+            prevCh = nextCh;
+            nextCh = UTF8toUTF32(str+i, &len);
+            w += GetAdvance(prevCh, nextCh);
+        }
+        //last character width
+        double asc, desc, width;
+        GetMetrics(nextCh, asc, desc, width);
+        w += width;
+        return w;
+    }
     
     /******** *nix specific ********/
 #ifndef __APPLE__
@@ -138,6 +164,16 @@ struct SSysFontInfo {
     typedef std::map<unsigned int,SCharMetric> TMetrics;
     TMetrics m_AFMCharMetrics;
     SCharMetric m_AFMFontBBox;
+
+    struct SCharPair {
+        unsigned int prev, next;
+        SCharPair(unsigned int p, unsigned int n) : prev(p), next(n) {}
+        friend bool operator< (const SCharPair &p1, const SCharPair &p2) {
+            return memcmp(&p1,&p2, sizeof(SCharPair)) < 0;
+        }
+    };
+    typedef std::map<SCharPair, double> TKerningTable;
+    TKerningTable m_AFMKerningTable;
 #endif
 #ifdef HAVE_XFT
     static Display *s_XDisplay; //global connection to X server
@@ -217,6 +253,12 @@ struct SSysFontInfo {
                                               XFT_WEIGHT_MEDIUM),
                  NULL);
             if (m_FontInfo) {
+                FT_Face face = XftLockFace(m_FontInfo);
+                if (m_Spec.m_Family != face->family_name) {
+                    Rf_warning("devEMF: your system substituted font family '%s' when you requested '%s'",
+                               face->family_name, m_Spec.m_Family.c_str());
+                }
+                XftUnlockFace(m_FontInfo);
                 return;
             } //otherwise proceed to AFM files
         }
@@ -225,10 +267,10 @@ struct SSysFontInfo {
         if (afmPathDB.find(m_Spec.m_Family) == afmPathDB.end()  ||
             afmPathDB[m_Spec.m_Family].size() < m_Spec.m_Face) {
 #ifdef HAVE_XFT
-            Rf_warning("Font metric information not found for family '%s'; "
+            Rf_warning("devEMF: font metric information not found for family '%s'; "
                        "using 'Helvetica' instead", m_Spec.m_Family.c_str());
 #else
-            Rf_warning("Font metric information not available for family '%s'; "
+            Rf_warning("devEMF: font metric information not available for family '%s'; "
                        "using 'Helvetica' instead (consider installing Xft through a system level-package called 'libxft-dev' or similar and then reinstall the devEMF package).", m_Spec.m_Family.c_str());
 #endif
 
@@ -264,6 +306,8 @@ struct SSysFontInfo {
 
 #ifdef HAVE_ZLIB
     void LoadAFM(const std::string &filename, int size, bool loadFontBBox) {
+        typedef std::map<std::string, unsigned int> TName2Code;
+        TName2Code name2code;
         const unsigned int buffsize = 512;
         char buff[buffsize];
         gzFile afm = gzopen(filename.c_str(), "rb");
@@ -298,6 +342,18 @@ struct SSysFontInfo {
                 if (m_AFMCharMetrics.find(cMetric.code) ==
                     m_AFMCharMetrics.end()) {
                     m_AFMCharMetrics[cMetric.code] = cMetric;
+                    name2code[cMetric.name] = cMetric.code;
+                }
+            } else if (key == "KPX") {
+                std::string name1, name2;
+                iss >> name1 >> name2;
+                TName2Code::const_iterator ch1 = name2code.find(name1);
+                TName2Code::const_iterator ch2 = name2code.find(name2);
+                if (ch1 != name2code.end()  &&
+                    ch2 != name2code.end()) {
+                    int kern;
+                    iss >> kern;
+                    m_AFMKerningTable[SCharPair(ch1->second, ch2->second)] = kern * 0.001 * size;
                 }
             }
         }
@@ -318,7 +374,7 @@ struct SSysFontInfo {
 #endif
     }
     
-#ifdef HAVE_FREETYPE
+#ifdef HAVE_XFT
     struct SPathOutlineFuncs : public FT_Outline_Funcs {
         SPathOutlineFuncs(void) {
             move_to = (FT_Outline_MoveToFunc) MoveTo;
@@ -362,7 +418,6 @@ struct SSysFontInfo {
 #endif
 
     void AppendGlyphPath(unsigned int c, EMFPLUS::SPath &path) const {
-#ifdef HAVE_FREETYPE
 #ifdef HAVE_XFT
         if (!m_FontInfo) {
             Rf_error("devEMF: font (%s) not found by Xft so can't embed fonts!",
@@ -387,31 +442,36 @@ struct SSysFontInfo {
         //Rprintf("totalpts: %d\n", path.m_TotalPts);
         return;
 #endif
-#endif
-        Rf_error("devEMF: Font to path conversion requires devEMF to be compiled with FreeType and Xft (probably you need to first install system level-packages called 'libfreetype-dev' and 'libxft-dev' and then reinstall the devEMF package).");
+        Rf_error("devEMF: Font to path conversion requires devEMF to be compiled with Xft and FreeType (perhaps you need to first install system level-packages called 'libxft-dev' and 'libfreetype-dev' and then reinstall the devEMF package).");
     }
 
     // actual advance (accounting for kerning!)
-    int GetAdvance(unsigned long prevC, unsigned long nextC) const {
-#ifdef HAVE_FREETYPE
+    double GetAdvance(unsigned long prevC, unsigned long nextC) const {
 #ifdef HAVE_XFT
-        if (!m_FontInfo) {
-            Rf_error("devEMF: font (%s) not found by Xft so can't embed fonts!",
-                     m_Spec.m_Family.c_str());
+        if (m_FontInfo) {
+            FT_Face face = XftLockFace(m_FontInfo);
+            //Rprintf("\nsize scaling:  %u, %i,%i, %f, %f\n", face->units_per_EM, face->size->metrics.x_ppem, face->size->metrics.y_ppem, face->size->metrics.x_scale/(double) 65536, face->size->metrics.y_scale / (double) 65536);
+            unsigned int prevI = FT_Get_Char_Index(face, prevC);
+            unsigned int nextI = FT_Get_Char_Index(face, nextC);
+            FT_Vector kerning;
+            FT_Get_Kerning(face, prevI, nextI, FT_KERNING_DEFAULT, &kerning);
+            XftUnlockFace(m_FontInfo);
+            FT_Load_Glyph(face, prevI, FT_LOAD_NO_BITMAP);
+            //Rprintf("kkkerning %u %u %f %f %f\n", prevC, nextC, face->glyph->advance.x/(double)64, kerning.x/(double)64, face->glyph->linearHoriAdvance/(double)65536);
+            return (face->glyph->advance.x + (face->glyph->lsb_delta - face->glyph->rsb_delta) + kerning.x)/(double)64;
         }
-        FT_Face face = XftLockFace(m_FontInfo);
-        FT_Set_Pixel_Sizes(face, m_Spec.m_Size, 0);
-        unsigned int prevI = FT_Get_Char_Index(face, prevC);
-        unsigned int nextI = FT_Get_Char_Index(face, nextC);
-        FT_Vector kerning;
-        FT_Get_Kerning(face, prevI, nextI, FT_KERNING_DEFAULT, &kerning);
-        XftUnlockFace(m_FontInfo);
-        FT_Load_Glyph(face, prevI, FT_LOAD_NO_BITMAP);
-        //Rprintf("kkkerning %u %u %i\n", prevC, nextC, kerning.x>>6);
-        return (face->glyph->advance.x>>6) + (kerning.x>>6);
 #endif
+        double wid = 0;
+#ifdef HAVE_ZLIB
+        double asc, desc;
+        GetMetrics(prevC, asc, desc, wid);
+        TKerningTable::const_iterator kernI = m_AFMKerningTable.find(SCharPair(prevC, nextC));
+        if (kernI != m_AFMKerningTable.end()) {
+             wid += kernI->second;
+             //Rprintf("kkkerning %u %u %f\n", prevC, nextC, kernI->second);
+        }
 #endif
-        Rf_error("devEMF: Font to path conversion requires devEMF to be compiled with FreeType and Xft (probably you need to first install system level-packages called 'libfreetype-dev' and 'libxft-dev' and then reinstall the devEMF package).");
+        return wid;
     }
     
     void GetMetrics(unsigned int c, 
@@ -439,78 +499,6 @@ struct SSysFontInfo {
             descent = m->second.descent;
             width = m->second.width;
         }
-#endif
-    }
-    double GetStrWidth(const char *str) const {
-#ifdef HAVE_XFT
-        if (m_FontInfo) {
-#ifdef HAVE_FREETYPE //freetype has kerning info
-            double w = 0;
-            unsigned int length = strlen(str);
-            unsigned char len1, len2;
-            unsigned char arr[4];
-            unsigned long ch1, ch2;
-            len2 = UTF8codepointBytes(str[0]);
-            memset(arr, 0, 4); memcpy(arr + 4-len2, str, len2);
-            arr[4-len2] &= 255 >> len2;
-            ch2 = (arr[0] & 63) << 18  | (arr[1] & 63) << 12 | (arr[2] & 63) << 6 | arr[3] & 127;
-            unsigned int i;
-            for (i = 0;  i+len2 < length;  i += len1) {
-                len1 = len2;
-                len2 = UTF8codepointBytes(str[i+len1]);
-                ch1 = ch2;
-                memset(arr, 0, 4); memcpy(arr + 4-len2, str+i+len1, len2);
-                arr[4-len2] &= 255 >> len2;
-                ch2 = (arr[0] & 63) << 18  | (arr[1] & 63) << 12 | (arr[2] & 63) << 6 | arr[3] & 127;
-                w += GetAdvance(ch1, ch2);
-            }
-            if (i < length) {//last width
-                XGlyphInfo extents;
-                FcChar32 c = ch2;
-                XftTextExtents32(s_XDisplay, m_FontInfo, &c, 1, &extents);
-                w += extents.xOff;
-            }
-            return w;
-#else //otherwise just extents
-            XGlyphInfo extents;
-            XftTextExtentsUtf8(s_XDisplay, m_FontInfo, (unsigned char*) str,
-                               strlen(str), &extents);
-            return extents.xOff;
-#endif
-        }
-#endif
-#ifdef HAVE_ZLIB
-        void *cd = Riconv_open("UCS-2", "UTF-8");
-        if (cd == (void*)(-1)) {
-            Rf_error("EMF device failed to convert UTF-8 to UCS-2");
-            return 0;
-        }
-        size_t inLeft = strlen(str);
-        const char *in = str;
-        size_t outLeft = inLeft*2;//overestimate
-        char *ucs2Str = new char[outLeft];
-        char *out = ucs2Str;
-        size_t res = Riconv(cd, &in, &inLeft, &out, &outLeft);
-        if (res != 0) {
-            Riconv_close(cd);
-            Rf_error("Text string not valid UTF-8");
-            delete[] ucs2Str;
-            return 0;
-        }
-        Riconv_close(cd);
-
-        double w = 0;
-        unsigned int len = strlen(str)*2 - outLeft;
-        for (unsigned int i = 0;  i < len;  i += 2) {
-            int c = ((unsigned char) ucs2Str[i]) +
-                (((unsigned char) ucs2Str[i+1]) << 8);
-            TMetrics::const_iterator m= m_AFMCharMetrics.find(c);
-            if (m != m_AFMCharMetrics.end()) {
-                w += m->second.width;
-            }
-        }
-        delete[] ucs2Str;
-        return w;
 #endif
     }
     void GetFontBBox(double &ascent, double &descent, double &width) {
@@ -553,7 +541,7 @@ std::string SSysFontInfo::packagePath;
         }
     };
     typedef std::map<SCharPair, int> TKerningTable;
-    mutable TKerningTable m_KerningTable;
+    TKerningTable m_KerningTable;
 
     SSysFontInfo(const SFontSpec& spec) : m_Spec(spec) {
         m_DC = GetDC(0);
@@ -576,6 +564,18 @@ std::string SSysFontInfo::packagePath;
                             lf.lfFaceName, LF_FACESIZE);
         m_FontHandle = CreateFontIndirect(&lf);
         SelectObject(m_DC, m_FontHandle);
+
+        unsigned int nPairs = GetKerningPairsW(m_DC, 0, NULL);
+        if (nPairs > 0) {
+            KERNINGPAIR *kernPairs = new KERNINGPAIR[nPairs];
+            GetKerningPairsW(m_DC, nPairs, kernPairs);
+            for (unsigned int i = 0;  i < nPairs;  ++i) {
+                m_KerningTable[SCharPair(kernPairs[i].wFirst,
+                                         kernPairs[i].wSecond)] =
+                    kernPairs[i].iKernAmount;
+            }
+            delete[] kernPairs;
+        }
     }
     ~SSysFontInfo(void) {
         DeleteObject(m_FontHandle);
@@ -646,20 +646,7 @@ std::string SSysFontInfo::packagePath;
         } // end of each contour
         delete[] buff;
     }
-    int GetAdvance(unsigned long prevC, unsigned long nextC) const {
-        if (m_KerningTable.empty()) {
-            unsigned int nPairs = GetKerningPairsW(m_DC, 0, NULL);
-            if (nPairs > 0) {
-                KERNINGPAIR *kernPairs = new KERNINGPAIR[nPairs];
-                GetKerningPairsW(m_DC, nPairs, kernPairs);
-                for (unsigned int i = 0;  i < nPairs;  ++i) {
-                    m_KerningTable[SCharPair(kernPairs[i].wFirst,
-                                             kernPairs[i].wSecond)] =
-                        kernPairs[i].iKernAmount;
-                }
-                delete[] kernPairs;
-            }
-        }
+    double GetAdvance(unsigned long prevC, unsigned long nextC) const {
         double ascent, descent, width;
         GetMetrics(prevC, ascent, descent, width);
         TKerningTable::const_iterator i =
@@ -686,25 +673,6 @@ std::string SSysFontInfo::packagePath;
             width = metrics.gmCellIncX;
         }
     }
-    double GetStrWidth(const char *str) const {
-        int nChar =
-            MultiByteToWideChar(CP_UTF8, 0, str, -1, NULL, 0);
-        WCHAR *wStr = new WCHAR[nChar];//nChar includes terminating null!
-        MultiByteToWideChar(CP_UTF8, 0, str, -1, wStr, nChar);
-        --nChar; //take off terminating null
-        double w = 0;
-        int i;
-        for (i = 0;  i+1 < nChar;  ++i) {
-            w += GetAdvance(wStr[i], wStr[i+1]);
-        }
-        if (i < nChar) { //get width of last character (no kerning)
-            double ascent, descent, width;
-            GetMetrics(wStr[i], ascent, descent, width);
-            w += width;
-        }
-        delete[] wStr;
-        return w;
-    }
     void GetFontBBox(double &ascent, double &descent, double &width) {
         TEXTMETRIC metrics;
         GetTextMetrics(m_DC, &metrics);
@@ -762,8 +730,17 @@ std::string SSysFontInfo::packagePath;
     void AppendGlyphPath(unsigned int c, EMFPLUS::SPath &path) const {
         Rf_error("devEMF: Font to path conversion not implemented for Apple; contact author.");
     }
-    int GetAdvance(unsigned long prevC, unsigned long nextC) const {
-        Rf_error("devEMF: Font to path kerning not implemented for Apple; contact author.");
+    double GetAdvance(unsigned long prevC, unsigned long nextC) const {
+        CGGlyph glyph[2];
+        UniChar ch[2];
+        ch[0] = prevC;
+        ch[1] = nextC;
+        CTFontGetGlyphsForCharacters (m_FontInfo, ch, glyph, 2);
+        CGSize adv[2];
+        CTFontGetAdvancesForGlyphs(m_FontInfo,
+                                   kCTFontOrientationDefault,
+                                   glyph, adv, 2);
+        return adv[0].width;
     }
     void GetMetrics(unsigned int c, 
                     double &ascent, double &descent, double &width) const {
@@ -780,24 +757,6 @@ std::string SSysFontInfo::packagePath;
                                            kCTFontOrientationDefault,
                                            &glyph, NULL, 1);
     }
-    double GetStrWidth(const char *str) const {
-        CFStringRef cfStr =
-            CFStringCreateWithCString(NULL, str, kCFStringEncodingUTF8);
-        int len = CFStringGetLength(cfStr);
-        CGGlyph* ch = new UniChar[len];
-        for (int i = 0;  i < len;  ++i) {
-            ch[i] = CFStringGetCharacterAtIndex(cfStr, i);
-        }
-        CGGlyph* glyphs = new CGGlyph[len];
-        CTFontGetGlyphsForCharacters (m_FontInfo, ch, glyphs, len);
-        double w = CTFontGetAdvancesForGlyphs(m_FontInfo,
-                                              kCTFontOrientationDefault,
-                                              glyphs, NULL, len);
-        delete[] glyphs;
-        delete[] ch;
-        CFRelease(cfStr);
-        return w;
-    }
     void GetFontBBox(double &ascent, double &descent, double &width) {
         ascent = CTFontGetAscent(m_FontInfo);
         descent = CTFontGetDescent(m_FontInfo);
